-- historically, the responsibilities for drawing resource
-- icons have been scattered over the code base.
--
-- this file is an attempt to consolidate and standardize some 
-- of that older functionality.  and it more or less follows
-- the more modern patterns established by
-- ImprovementIcons.lua and TechGlyphs.lua.
--
-- there are still a few important eccentricities, however. 
-- namely, many, but not all, of the resource icons used in the
-- top bar are hand-tuned pixel art assets, and as such, are
-- managed by GUI\FancyIcons\tiny_icons.
--
-- for all resource icons, there are between 1 and 4 different
-- contexts in which it may be displayed.
--
-- 1) as a Glyph, inside a formatted text block. 
--
-- 2) as an element of the top bar, in which case, it will
-- prefix some text element.
--
-- 3) as an element of the planet resource production pane, in
-- which case, it will be a suffix to one of the column of
-- numbers. 
-- 
-- 4) wrench and metal icons appear a second time in the
-- production pane, as a suffix to a progress bar display in
-- the bottom pane.
--
-- The icon configuration process creates special purpose
-- callback functions for all declared glyphs that handle all 4
-- cases, though many of these will never be used.
--
-- We also define the standard ResourceGlyph.tag_sub symbol
-- substitution rules, which, taken together with the trickery
-- in symbols.lua, allows desc_format text blocks to use text
-- like '<COIN>' as shorthand for certain special glyphs.
--
--
_ENV = gui
local dir = current_lua_dir()
local char in string


ResourceGlyph = ResourceGlyph or {}

--[[
local function h_scaled_vimage(fname, x,y,h)
	local iw,ih = vimage_dims(fname)

	local s = h/ih

--	scale_fun(s,draw_vimage,x/s,y/s,fname)

	return x,y,s*iw,h

end--]]

-- 
-- the semantics for each glyphs arguments are a bit odd.
-- the basic problem is that there are particular quirks to all
-- 4 potential display cases.  for example, glyphs shown in the
-- top bar generally ought to be drawn a bit larger, relative
-- to the surrounding text, than glyphs inserted into formatted
-- description text.  those sorts of special-case tweaks
-- require adding extra parameters to the glyph definitions --
-- and the system has thus grown organically into a bit of a
-- mess.  the parameter semantics could stand to be cleaned up,
-- but, for now, i like the way things look well enough -- so
-- i'm not going to keeping messing with it.
local arg_inits = ARGS_OR { pre=0, post = .2, hs = 1.3, ds = .3, top_bar_scale=1 }

local function make_vimage_glyph(args)
  local hs,ds,pre,post,top_bar_scale in arg_inits(args)

  local img = args.img or (dir..args!.src..'.png')
  local color in args

  local rval = {..img, ..color}

  -- used to display icons in the top bar
  rval.top_bar_str = restore_color | function(font, pt, x, y, str)
    local hs,ds=hs,ds
    hs,ds*=top_bar_scale
    if color
      set_frame_color4 | color()
    end
    local _,y0,cw,h0 = h_scaled_vimage(img,x+pt*pre,y-pt*ds,pt*hs)
    set_frame_color4(1,1,1,1)
    local x,y,w,h = display_text( font ,pt, x+cw+pt*post,y,str)

    return x-cw-pre*pt,y0,w+cw+pt*(pre+post),h0 
  end

  -- used by the wrench suffix drawing in the bottom of the
  -- production pane.
  rval.suffixed = restore_color | function(font, pt, x, y, str)
--    glColor4d(0,0,0,0)
    
    if SCOPE.font_color
      set_frame_color4 | SCOPE.font_color()
    end

    local x,y,w,h = display_text( font, pt, x, y, str )
 --   glColor4d(1,1,1,1)
    if color
      set_frame_color4 | color()
    end

    local _,y0,cw,h0 = h_scaled_vimage(img,x+pt*pre+w,y-pt*ds,pt*hs)
    --local advance = SCOPE_call(icon_radius_hint=pt,icon.simple_draw,x+w+pt*.4,y0+pt*.35)
    return x,y,w+cw+pt*(pre+post),h0
  end



  -- used to display resource icons in the top of the planet
  -- production pane.
  rval.column = restore_color | function(font, pt, x, y, str)
--    glColor4d(0,0,0,0)
    local x,y,w,h = br_text( font, pt, x, y, str )
 --   glColor4d(1,1,1,1)
    if color
      set_frame_color4 | color()
    end

    local _,y0,cw,h0 = h_scaled_vimage(img,x+pt*pre+w,y-pt*ds,pt*hs)
    --local advance = SCOPE_call(icon_radius_hint=pt,icon.simple_draw,x+w+pt*.4,y0+pt*.35)
    return x,y,w+cw+pt*(pre+post),h0
  end

  local r,g,b,a=(color or white)()
  local ds = args.glyph_ds or ds
  local pre = args.glyph_pre or pre

  -- used to display icons inside formatted text
  local draw_glyph = restore_color | function(fh,x,y)

    set_frame_color4 (r,g,b,a)

    --local _TRACE_INFO=img
		local _,y0,cw,h0 = h_scaled_vimage(img,x+fh*pre,y-fh*ds,fh*hs)

		return x,y0,cw+pre*fh,h0    
  end

  local kerning = args.kerning?

  insert(rval,..draw_glyph)

  function rval.typeset_callback(x,y,prev,str,post,font,pt)

    local advance=args!.base_advance

		local kern = kerning?[ char(prev) ]
		if kern
			x-=kern*pt*64
			advance-=kern
		end

		if kerning.xshift
			x-=kerning.xshift*pt*64
		end

		x/=64

		return advance*pt*64, function()
			draw_glyph(pt,x,y)
		end

  end

  return rval
end




ResourceGlyph.metal = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.1, post=.3, src='minerals',
  top_bar_scale=1.2,

  base_advance=1.2,
  kerning = {['(']=.12},

}

ResourceGlyph.vwrench = make_vimage_glyph {
  hs=1.3,ds=.32,pre=.13,post=.3,src='vwrench_cutout',
  base_advance = .4,
  kerning={xshift=0},
}

--[[
--]]


--[[
ResourceGlyph.food = make_vimage_glyph {
  hs=1.4,ds=.45,pre=0,post=.1,src='ecology',
  top_bar_scale=1.2,

  base_advance=1.0,
  kerning = {['(']=.12},
}--]]

ResourceGlyph.food = make_vimage_glyph {
  hs=1.2,ds=.2,pre=.0, post=.1,src='food',
  
  glyph_pre=.15, glyph_ds=.15,
  top_bar_scale=1.2,


  base_advance=1.1,
  kerning = {['(']=.12},
}



ResourceGlyph.corn = make_vimage_glyph {
  hs=1.25,ds=.3,pre=-.05, post=.0,
  img=[[Lua state\GUI\FancyIcons\notification_icons\misc\corn_5.png]],
  
  --glyph_pre=.15, glyph_ds=.2,
  top_bar_scale=1.25,


  base_advance=1.1,
  kerning = {['(']=.12},
}


ResourceGlyph.corn3 = make_vimage_glyph {
  hs=1.1,ds=.18,pre=.00, post=-.1,
  img=[[Lua state\GUI\FancyIcons\notification_icons\misc\corn_3.png]],
  
  --glyph_pre=.15, glyph_ds=.2,
  top_bar_scale=1.25,


  base_advance=1.1,
  kerning = {['(']=.12},
}

ResourceGlyph.coin = make_vimage_glyph {
  hs=1.2,ds=.25,pre=.15,post=.12,src='coin0',
  top_bar_scale=1.2,
  color = catch(1,.9,0,1),
  base_advance=.8,
  kerning = {['(']=.12},
}

ResourceGlyph.wrench = make_vimage_glyph {
  hs=1.2,ds=.3,pre=.1,post=.12,src='wrench_II',
  top_bar_scale=1.2,
  color = catch(1,.9,0,1),
  base_advance=1.2,
  kerning = {xshift=.1},
}

ResourceGlyph.beaker = make_vimage_glyph {
  hs=1.25,ds=.3,pre=.15,post=0,src='beaker1',
  -- 176
  color = color_param 'hsv: 182; 100; 100',
  base_advance = .4,
  kerning = {xshift=.08,}, 
}



--print | ( color_param 'hsv: 182; 100; 100')()

ResourceGlyph.cargo_old = make_vimage_glyph {
  hs=1.2, ds=.2, pre=0.05, post=0.05, src='cargo',
  top_bar_scale=1.2,
  base_advance=.8,
  kerning = {['(']=.0},
}

ResourceGlyph.cargo = make_vimage_glyph {
  hs=1.2, ds=.25, pre=0.1, post=0.05, 
  
  img=[[Lua state\GUI\FancyIcons\notification_icons\misc\cargo.png]], 
  --color = color_param 'ffb653',
  color = color_param 'hsv: 35; 80; 100', -- increased jim's suggested saturation value a bit -- keeping the same hue and lightness though.

  top_bar_scale=1.2,
  base_advance=1.2,
  kerning = {['(']=.0},
}

--print(format_as_hsv | color_param 'ffb653' () )


ResourceGlyph.transport = make_vimage_glyph {
  hs=1.3, ds=.4, pre=0.05, post=0.05, src='transport',
  top_bar_scale=1.2,
  base_advance=1.2,
  kerning = {['(']=.0},
}

ResourceGlyph.ingot_pile = make_vimage_glyph {
  hs=1.3, ds=.4, pre=0.05, post=0.05, src='ingot_pile',
  top_bar_scale=1.2,
  base_advance=1.2,
  kerning = {['(']=.0},
}

ResourceGlyph.smile = make_vimage_glyph {
  hs=1, ds=.2, pre=0.2, post=0.05, src='smile',
  -- 176
--  color = color_param 'c896ff',
    color = color_param '00aaff',

  base_advance = 1.2,
  kerning ={['(']=.0}, 
}

ResourceGlyph.frown = make_vimage_glyph {
  hs=1, ds=.2, pre=0.2, post=0.05, src='frown',
  -- 176
  color = color_param 'ff0000',
  base_advance = 1.2,
  kerning ={['(']=.0}, 
}

ResourceGlyph.ingot = make_vimage_glyph {
  hs=1.0, ds=.2, 
  pre=0.13, post=0.0, src='ingot',

  base_advance=1.35,
  kerning = {['(']=.0},
}

ResourceGlyph.handshake = make_vimage_glyph {
  hs=0.9, ds=.1, 
  top_bar_scale=1.3,

  pre=0.13, post=0.3, src='handshake',

  base_advance=1.65,
  kerning = {['(']=.0},
}

ResourceGlyph.artifacts = make_vimage_glyph {
  hs=1.4,ds=.35, pre=0,post=.1,src='artifacts',
  top_bar_scale=1.2,

  base_advance=1.6,
--  kerning = {['(']=.12},
}


ResourceGlyph.opil = make_vimage_glyph {
  hs=1.4,ds=.35, pre=.2,post=.1,src='opil',
  top_bar_scale=1.2,
  base_advance=1.1,


--  kerning = {['(']=.12},
}

ResourceGlyph.debris_ring = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.1, post=.3, src='asteroids',
  top_bar_scale=1.2,

  base_advance=1.3,
  kerning = {['(']=.12},
--  kerning = {['(']=.12},
}

ResourceGlyph.rare_earths = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.1, post=.3, src='minerals',
  top_bar_scale=1.2,

  base_advance=1.2,
  kerning = {['(']=.12},
--  kerning = {['(']=.12},
}

ResourceGlyph.mineral_rich = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.1, post=.3, src='rich',
  top_bar_scale=1.2,

  base_advance=1.3,
  kerning = {['(']=.12},
--  kerning = {['(']=.12},
}

ResourceGlyph.mineral_poor = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.1, post=.3, src='shovel',
  top_bar_scale=1.2,

  base_advance=1.3,
  kerning = {['(']=.12},
--  kerning = {['(']=.12},
}

ResourceGlyph.unrest = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.1, post=.3, src='war_sword',
  top_bar_scale=1.2,

  base_advance=1.3,
  kerning = {['(']=.12},
--  kerning = {['(']=.12},
}

ResourceGlyph.gems = make_vimage_glyph {
  hs=1.4, ds=.4, pre=.15, post=.3, src='gem',
  top_bar_scale=1.2,

  base_advance=1.9,
--  kerning = {['(']=.12},
}

ResourceGlyph.super_grain = make_vimage_glyph {
  hs=1.8,ds=.45,pre=0,post=.1,src='grain',
  top_bar_scale=1.2,

  base_advance=1.0,
  kerning = {['(']=.12},
}

local subs = {}

for icon, abbrev ; {  
  corn3 = "FOOD", ingot='METAL', cargo='CARGO', transport='TRANSPORT', 
  wrench='WRENCH', coin='COIN', beaker = "BEAKER",handshake='INFLUENCE',
  smile = 'MORALE', frown='DISCONTENT', 
  
  artifacts='ARTIFACTS',opil='OPIL',
  rare_earths='RARE_EARTHS', gems='GEMS', super_grain='SUPER_GRAIN',
	debris_ring='DEBRIS_RING',
  
  }
  
  subs[abbrev] =escape_string | '~'..icon
end

function ResourceGlyph.tag_sub(code)
  return subs[code]
end


-- these special functions are referenced in various places.
function coin_symbol()
	return char(27).."~coin"..char(27)
end

function beaker_symbol()
	return char(27).."~beaker"..char(27)
end

function hammer_symbol()
	return char(27).."~wrench"..char(27)
end

function metal_symbol()
	return char(27).."~ingot"..char(27)
end


